
% MATLAB - Kurzeinfhrung
% -----------------------

% MATLAB ist ein kommerzielles Numerik-Programmpaket, das Studierenden der 
% Universitt Stuttgart zur Verfgung steht:
% https://www.tik.uni-stuttgart.de/dienste-a-z/software-und-lizenzen-fuer-studierende/

% MATLAB verwendet eine numerische Darstellung von Vektoren und Matrizen 
% und erlaubt eine einfache Programmierung von Vektor/Matrixoperationen und 
% stellt eine Vielzahl von Methoden zur Systemanalyse zur Verfgung. Viele 
% so genannte Toolboxen sind verfgbar, welche die Funktionalitt von 
% MATLAB erweitern. Die Befehle knnen direkt in dem MATLAB-Fenster 
% eingegeben und ausgefhrt werden. Alternativ knnen die Befehle auch in 
% einer Textdatei <name>.m zusammengefasst werden. Dieses so genannte 
% m-file wird durch Eingabe von <name> in dem MATLAB-Fenster geladen und 
% ausgefhrt.

doc                             % startet die Online-Hilfe
a = [ 3 -4 5 ]                  % definiert einen Zeilenvektor 
b = [ 3 -4 5 ]'                 % definiert einen Spaltenvektor
A = [ 1 2 4 
2 13 23 
4 33 77 ]	% definiert eine Matrix
A'                              % Transponierte der Matrix A
A(2, 3)                         % Matrixelement A23
A(:, 3)                         % 3. Spalte der Matrix A
A(1, :)                         % 1. Zeile der Matrix A
zeros(3, 4)                     % definiert eine Nullmatrix
ones(3, 4)                      % definiert eine Matrix mit Einsern
diag([1 2 3])                   % definiert eine Diagonalmatrix
c = A*b                         % Matrix-Vektor Multiplikation
c = A\b                         % lst ein lineares Gleichungssystem
inv(A)                          % Inverse einer Matrix
[ EV, ew ] = eig(A)             % Matrix EV der Eigenvektoren, Vektor ew 
                                % der Eigenwerte der Matrix A
plot(a, b, '--r'), hold on;     % Plottet Vektor b ber Vektor a
whos                            % Zeigt alle im Workspace belegten Variablen


% Numerischen Zeitintegration
% ---------------------------

% Zur numerischen Zeitintegration stellt MATLAB zwei explizite 
% Einschrittverfahren (ode23, ode45), ein implizites Einschrittverfahren 
% (ode23s), ein Adams-Bashforth-Moulton Prdiktor-Korrektor-Verfahren 
% (ode113) und eine implizite 'numerical differentiation formula' (ode15s) 
% zur Verfgung. Alle Verfahren haben eine eingebaute Schrittweitensteuerung. 

% [t,X] = ode23('<name>',tspan,x0)      
% simuliert ein System gewhnlicher Differentialgleichungen der Form 
% dx/dt = f(t,x). Ausgabe ist der Vektor t der Zeitschritte und die Matrix 
% X der Zustandstrajektorien. Die Anzahl der Spalten von X entspre-chen der 
% Anzahl der Zustnde und die Anzahl der Zeilen entspricht der Anzahl der 
% Zeitschritte. Eingabe ist mit <name> eine Funktion dx/dt, ein (2x1) 
% Vektor tspan = [t0, tend] welcher die Anfangs- und Endzeit enthlt und 
% der Vektor der Anfangsbedingungen x0 = [ ].	Die Funktion <name> muss in 
% einer extra Datei <name.m> mit entsprechendem Dateiname und folgender 
% Syntax gespeichert sein:

% function dx = doublependulum(t, x)
% ...
% dx = ...

% Beispiel: Doppelpendel


[T, Y] = ode45(@doublependulum, [0 3], [0.2 0.2 0 0]);
plot(T, Y)

%   function dx = doublependulum(t, x)
%     % state vector x = [alp, bet, Dalp, Dbet]';
%     alp = x(1); 
%     bet = x(2);
%     Dalp = x(3); 
%     Dbet = x(4);
%     % mass and geometry parameters
%     m = 1.0; l = 1.0; g = 9.81;
%     
%     % mass matrix M
%     M = zeros(2, 2);
%     M(1, 1) = l^2*m*(3+2*cos(bet));    
%     M(1, 2) = l^2*m*(cos(bet)+1);
%     M(2, 1) = l^2*m*(cos(bet)+1); 
%     M(2, 2) = l^2*m;
%     
%     k = zeros(2, 1);
%     k(1, 1) = -l^2*m*Dbet*sin(bet)*(Dbet+2*Dalp);
%     k(2, 1) = l^2*m*Dalp^2*sin(bet);
%     
%     q = zeros(2, 1);
%     q(1, 1) = -m*g*l*(2*cos(alp)+cos(alp+bet));
%     q(2, 1) = -cos(alp+bet)*l*m*g;
% 
% 	dx = [[Dalp; Dbet]; inv(M)*(q-k)]; numerisch nicht effizient
%   dx = [[Dalp; Dbet]; M\(q-k)];
 



% MATLAB Symbolic Toolbox
% -----------------------

% Mit der Symbolic Toolbox knnen in MATLAB neben numerischen Berechnungen 
% auch symbolische Untersuchungen durchgefhrt werden. Bis zu der 
% MATLAB-Version R2008b basiert die Symbolic Toolbox auf dem MAPLE-Kernel, 
% ab R2009a basiert diese auf MuPAD. In beiden Versionen sind folgende 
% Grundfunktionen enthalten:

syms a b        % definieren symbolischer Variablen
c = a+b         % Grundrechenarten			
d = a*b
a = 5; b = 2		% Werte zuweisen
e = subs(c)  		% Ersetzen aller symbolischen Ausdrcke 
f = subs(c, b)	%	Ersetzen von b in c
 
% Beispiel: Berechnung von Jacobi-Matrix und Beschleunigungsvektor 
	
syms l1 alpha alpha_ Dalpha D2alpha beta Dbeta D2beta t_
 
% Vektor der verallgemeinerten Koordinaten und Ableitungen
y = [alpha; beta]
Dy = [Dalpha; Dbeta]
D2y = [D2alpha; D2beta]
 	
% Orstvektor
r1 = [l1*cos(alpha); l1*sin(alpha); 0]

% Zeitabhngige Variablen als Taylorreihe darstellen
alpha_ = alpha + Dalpha*t_ + 0.5*D2alpha*t_^2
 
% Berechnung der Geschwindigkeit v1 und Beschleunigung a1
r1_ = subs(r1, alpha, alpha_)
v1_ = diff(r1_, 't_')
v1 = subs(v1_, t_, 0)
 
a1_ = diff(v1_, 't_')
a1 = subs(a1_, t_, 0)
 
% Jacobi-Matrix und lokale Beschleunigung
J1 = jacobian(r1, y)
a1q = a1 - J1*D2y
 
